@@ -1,10 +1,16 @@
-/*
- * TODO: better heuristic (https://bugzilla.gnome.org/show_bug.cgi?id=623635#c5)
- * TODO: ProgressCallback types (not specific to async).
- * TODO: add annotation for methods like g_file_replace_contents_bytes_async where the finish
- * method has a different prefix.
- */
+// TODO: better heuristic (https://bugzilla.gnome.org/show_bug.cgi?id=623635#c5)
+// TODO: ProgressCallback types (not specific to async).
+// TODO: add annotation for methods like g_file_replace_contents_bytes_async
+// where the finish method has a different prefix.
 
+use std::{
+    borrow::Borrow,
+    collections::{HashMap, HashSet},
+};
+
+use log::warn;
+
+use super::{namespaces::NsId, special_functions};
 use crate::{
     analysis::{
         self,
@@ -31,17 +37,11 @@ use crate::{
     traits::*,
     version::Version,
 };
-use log::warn;
-use std::{
-    borrow::Borrow,
-    collections::{HashMap, HashSet},
-};
-
-use super::{namespaces::NsId, special_functions};
 
 #[derive(Clone, Debug)]
 pub struct AsyncTrampoline {
     pub is_method: bool,
+    pub has_error_parameter: bool,
     pub name: String,
     pub finish_func_name: String,
     pub callback_type: String,
@@ -55,7 +55,7 @@ pub struct AsyncFuture {
     pub is_method: bool,
     pub name: String,
     pub success_parameters: String,
-    pub error_parameters: String,
+    pub error_parameters: Option<String>,
     pub assertion: SafetyAssertionMode,
 }
 
@@ -235,6 +235,7 @@ fn fixup_gpointer_parameter(
     env: &Env,
     type_tid: library::TypeId,
     is_boxed: bool,
+    in_trait: bool,
     parameters: &mut Parameters,
     idx: usize,
 ) {
@@ -264,7 +265,7 @@ fn fixup_gpointer_parameter(
                 " as {}",
                 nameutil::use_glib_if_needed(env, "ffi::gconstpointer")
             ),
-            in_trait: false,
+            in_trait,
             nullable: false,
             move_: false,
         },
@@ -276,6 +277,7 @@ fn fixup_special_functions(
     name: &str,
     type_tid: library::TypeId,
     is_boxed: bool,
+    in_trait: bool,
     parameters: &mut Parameters,
 ) {
     // Workaround for some _hash() / _compare() / _equal() functions taking
@@ -284,7 +286,7 @@ fn fixup_special_functions(
         && parameters.c_parameters.len() == 1
         && parameters.c_parameters[0].c_type == "gconstpointer"
     {
-        fixup_gpointer_parameter(env, type_tid, is_boxed, parameters, 0);
+        fixup_gpointer_parameter(env, type_tid, is_boxed, in_trait, parameters, 0);
     }
 
     if (name == "compare" || name == "equal" || name == "is_equal")
@@ -292,8 +294,8 @@ fn fixup_special_functions(
         && parameters.c_parameters[0].c_type == "gconstpointer"
         && parameters.c_parameters[1].c_type == "gconstpointer"
     {
-        fixup_gpointer_parameter(env, type_tid, is_boxed, parameters, 0);
-        fixup_gpointer_parameter(env, type_tid, is_boxed, parameters, 1);
+        fixup_gpointer_parameter(env, type_tid, is_boxed, in_trait, parameters, 0);
+        fixup_gpointer_parameter(env, type_tid, is_boxed, in_trait, parameters, 1);
     }
 }
 
@@ -336,8 +338,8 @@ fn analyze_callbacks(
     let mut to_remove = Vec::new();
 
     {
-        // When closure data and destroy are specified in gir, they don't take into account the
-        // actual closure parameter.
+        // When closure data and destroy are specified in gir, they don't take into
+        // account the actual closure parameter.
         let mut c_parameters = Vec::new();
         for (pos, par) in parameters.c_parameters.iter().enumerate() {
             if par.instance_parameter {
@@ -587,8 +589,8 @@ fn analyze_function(
             .iter()
             .any(|par| par.c_type == "GDestroyNotify")
     {
-        // In here, We have a DestroyNotify callback but no other callback is provided. A good
-        // example of this situation is this function:
+        // In here, We have a DestroyNotify callback but no other callback is provided.
+        // A good example of this situation is this function:
         // https://developer.gnome.org/gio/stable/GTlsPassword.html#g-tls-password-set-value-full
         warn_main!(
             type_tid,
@@ -699,7 +701,14 @@ fn analyze_function(
         }
     }
 
-    fixup_special_functions(env, name.as_str(), type_tid, is_boxed, &mut parameters);
+    fixup_special_functions(
+        env,
+        name.as_str(),
+        type_tid,
+        is_boxed,
+        in_trait,
+        &mut parameters,
+    );
 
     // Key: destroy callback index
     // Value: associated user data index
@@ -708,10 +717,28 @@ fn analyze_function(
 
     if status.need_generate() {
         if !has_callback_parameter {
+            let mut to_remove = Vec::new();
+            let mut correction_instance = 0;
+            for par in parameters.c_parameters.iter() {
+                if par.scope.is_none() {
+                    continue;
+                }
+                if let Some(index) = par.user_data_index {
+                    to_remove.push(index);
+                }
+                if let Some(index) = par.destroy_index {
+                    to_remove.push(index);
+                }
+            }
             for (pos, par) in parameters.c_parameters.iter().enumerate() {
-                // FIXME: It'd be better if we assumed that user data wasn't gpointer all the time so
-                //        we could handle it more generically.
-                if r#async && is_gpointer(&par.c_type) {
+                if par.instance_parameter {
+                    correction_instance = 1;
+                }
+
+                if r#async
+                    && pos >= correction_instance
+                    && to_remove.contains(&(pos - correction_instance))
+                {
                     continue;
                 }
                 assert!(
@@ -879,11 +906,13 @@ fn analyze_function(
         }
 
         imports.add_used_types(&used_types);
-        if ret.base_tid.is_some() || parameters.c_parameters.iter().any(|p| p.move_) {
-            imports.add("glib::object::Cast");
+        if ret.base_tid.is_some() {
+            imports.add("glib::prelude::*");
         }
 
-        if func.name.parse::<special_functions::Type>().is_err() {
+        if func.name.parse::<special_functions::Type>().is_err()
+            || parameters.c_parameters.iter().any(|p| p.move_)
+        {
             imports.add("glib::translate::*");
         }
         bounds.update_imports(imports);
@@ -1004,19 +1033,13 @@ fn analyze_async(
             *commented = true;
             return false;
         }
-        if !*commented && (success_parameters.is_empty() || error_parameters.is_empty()) {
+        if !*commented && success_parameters.is_empty() {
             if success_parameters.is_empty() {
                 warn_main!(
                     type_tid,
                     "{}: missing success parameters for async future",
                     func.name
                 );
-            } else if error_parameters.is_empty() {
-                warn_main!(
-                    type_tid,
-                    "{}: missing error parameters for async future",
-                    func.name
-                );
             }
             *commented = true;
             return false;
@@ -1025,7 +1048,8 @@ fn analyze_async(
 
         *trampoline = Some(AsyncTrampoline {
             is_method,
-            name: format!("{}_trampoline", codegen_name),
+            has_error_parameter: error_parameters.is_some(),
+            name: format!("{codegen_name}_trampoline"),
             finish_func_name: format!("{}::{}", env.main_sys_crate_name(), finish_func_name),
             callback_type,
             bound_name,
@@ -1122,16 +1146,17 @@ fn analyze_callback(
             }
         }
 
-        // If we don't have a "user data" parameter, we can't get the closure so there's nothing we
-        // can do...
+        // If we don't have a "user data" parameter, we can't get the closure so there's
+        // nothing we can do...
         if par.c_type != "GDestroyNotify"
             && (func.parameters.is_empty() || !func.parameters.iter().any(|c| c.closure.is_some()))
         {
             *commented = true;
             warn_main!(
                 type_tid,
-                "Closure type `{}` doesn't provide user data",
-                par.c_type
+                "Closure type `{}` doesn't provide user data for function {}",
+                par.c_type,
+                func_name,
             );
             return None;
         }
@@ -1166,7 +1191,9 @@ fn analyze_callback(
             .direction(ParameterDirection::Return)
             .try_build()
         {
-            if !rust_type.as_str().ends_with("GString") {
+            if !rust_type.as_str().ends_with("GString")
+                && !rust_type.as_str().ends_with("GAsyncResult")
+            {
                 imports_to_add.extend(rust_type.into_used_types());
             }
         }
